function [V,D] = clustEigsExt(A,prtc,K,str)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of a square symmetric matrix
% A using SVDs of all dense blocks Aij.
%
% [V,D] = clustEigsExt(A,K,prtc,str)
%
% Input:
% A     - Sparse square non-symmetric matrix 
% prtc  - Cell-array containing vertices for each cluster
% K     - Matrix indicating ranks in the approximations of blocks Aij
%          If K(i,j) = 0, then Aij is not dense, thus not approximated.
%          If K(i,j) = k_ij > 0, then k_ij is the rank in the 
%          approximation of block Aij.
% str   - String, must be 'eigs' or 'laneig' to indicate algorithm 
%
% Output:
% V     - Cell-array with orthonormal matrices Vi. Each Vi contains 
%         contributions from all dense blocks in the i'th block-row.
% D     - Cell-array with Dij = Vi'*Aij*Vj
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------


% number of clusters
noc = length(prtc);    
rr = sum(K,2);

V = cell(noc,1);
for i = 1:noc
    V{i} = zeros(length(prtc{i}),rr(i));
end
rind = zeros(noc,1);

switch str
    case 'eigs'
        disp('Computing clustered matrix approximation with EIGS and SVDS')
        for i = 1:noc
            [VV,~] = eigs(A(prtc{i},prtc{i}),K(i,i));
            V{i}(:,rind(i)+1:rind(i) + K(i,i)) = VV;
            for j = i+1:noc
                if K(i,j) > 0
                    [UU,~,VV] = svds(A(prtc{i},prtc{j}),K(i,j));
                    V{i}(:,rind(i)+1:rind(i) + K(i,j)) = UU;
                    V{j}(:,rind(j)+1:rind(j) + K(i,j)) = VV;
                    rind(i) = rind(i) + K(i,j);
                    rind(j) = rind(j) + K(i,j);
                end
            end
        end
    case 'laneig'
        disp('Computing clustered matrix approximation with LANEIG and LANSVD')
        for i = 1:noc
            [VV,~] = laneig(A(prtc{i},prtc{i}),K(i,i));
            V{i}(:,rind(i)+1:rind(i) + K(i,i)) = VV;
            for j = i+1:noc
                if K(i,j) > 0
                    [UU,~,VV] = lansvd(A(prtc{i},prtc{j}),K(i,j));
                    V{i}(:,rind(i)+1:rind(i) + K(i,j)) = UU;
                    V{j}(:,rind(j)+1:rind(j) + K(i,j)) = VV;
                    rind(i) = rind(i) + K(i,j);
                    rind(j) = rind(j) + K(i,j);
                end
            end
        end
    otherwise
        disp('Valid string options are "svds" and "lansvd".')
end
for i = 1:noc
    [V{i}, ~] = qr(V{i},0);
end

D = compCoreSymFull(A,V,[],prtc,'dense');





