function [U,S,varargout] = clustRandAlg(A,prtc,k,p,str)
%--------------------------------------------------------------------------
% Compute clustered low rank approximation using randAlg.m
% 
% [V, D]    = clustRangAlg(A,prtc,k,p,'sym');          % A symmetric
% [U, S, V] = clustRandAlg(A,prtc,k,p,'non-sym');      % A non-symmetric
%
% Input:
% A     - Sparse symmetric matrix 
% prtc  - Cell-array containing vertices for each cluster
% k     - Scalar or vector indicating rank of cluster approximations
%         If k is scalar then the same k is used for all clusters. 
%         If k is vector, then length(k) must equal number of clusters.
% p     - Scalar or vector indicating the oversampling parameter
%         If p is scalar then the same p is used for all clusters. 
%         If p is vector, then length(p) must equal number of clusters.
%         The rank of the approximation of cluster i becomes k(i) + p(i).
% str   - string, must be 'sym' or 'non-sym'
%         If A is symmetric, use 'sym' and two output arguments.
%         If A is non-symmetric, use 'non-sym' and three output arguments.
%
% Output for symmetric A:
% V     - Cell-array with approximations Vi of the dominant k(i)+p(i) 
%         dimensional column space
% D     - Cell-array with Di = Vi'*Aii*Vi 
%
% Output for non-symmetric A:
% U     - Cell-array with approximations Ui of the dominant k(i)+p(i) 
%         dimensional column space
% S     - Cell-array with Di = Ui'*Aii*Vi 
% V     - Cell-array with approximations Vi of the dominant k(i)+p(i) 
%         dimensional row space
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

noc = length(prtc);     % number of clusters
U = cell(noc,1);        
S = cell(noc,1);

if length(k) == 1
    k = ones(noc,1)*k;
end

if length(p) == 1
    p = ones(noc,1)*p; 
end

switch str
    case 'sym'      
        for i = 1:noc
            disp(['Low rank computation of symmetric A with randAlg for cluster ' int2str(i), ' of ' int2str(noc)])
            U{i} = randAlg(A(prtc{i},prtc{i}),k(i),p(i),'orth');
            S{i} = U{i}'*A(prtc{i},prtc{i})*U{i};
        end
    case 'non-sym'
    V = cell(noc,1);
        for i = 1:noc
            disp(['Low rank computation of non-symmetric A with randAlg for cluster ' int2str(i), ' of ' int2str(noc)])
            U{i} = randAlg(A(prtc{i},prtc{i}),k(i),p(i),'orth');
            [V{i},S{i}] = qr((A(prtc{i},prtc{i})')*U{i},0);
        end
        varargout(1) = {V};
    otherwise 
        disp('Valid string options are "sym" and "non-sym".')
end

