function [U,S,V] = clustRandAlgRect(A,K,P,Q,rprtc,cprtc)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of a rectangular matrix A using 
% randomized algorithm (randAlgPow.m)
%
% [U,S,V] = clustRandAlgRect(A,K,P,Q,rprtc,cprtc)
%
% Input:
% A     - Sparse rectangular matrix 
% K     - Matrix indicating ranks in the approximations of blocks Aij
%          If K(i,j) = 0, then Aij is not dense, thus not approximated.
%          If K(i,j) = k_ij > 0, then k_ij is the rank in the 
% P     - Matrix with oversampling paramters for each dense block of A.
% Q     - Matrix with power parameters for each dense block of A.
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
%
% Output:
% U     - Cell-array with blocks of U = diag(U_1,...,U_r)
% S     - Cell-array with the S_ij = U_i'*A_ij*V_j
% V     - Cell-array with blocks of V = diag(V_1,...,V_c)
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of row and column clusters
rnoc = length(rprtc); 
cnoc = length(cprtc); 

rr = sum(K,2) + sum(P,2);
cc = sum(K,1) + sum(P,1);

U = cell(rnoc,1);
V = cell(cnoc,1);

for i = 1:rnoc
    U{i} = zeros(size(rprtc{i},1),rr(i));
end

for i = 1:cnoc
    V{i} = zeros(size(cprtc{i},1),cc(i));
end

rind = zeros(rnoc,1);
cind = zeros(cnoc,1);

disp('Computing clustered low rank approximation with Random approach')
for i = 1:rnoc
    for j = 1:cnoc
        if K(i,j) > 0
            disp(['Computation for block: ' int2str(i),' x ',...
                int2str(j),' of ', int2str(rnoc), ' x ', int2str(cnoc)])
            UU = randAlgPow(A(rprtc{i},cprtc{j}), K(i,j), P(i,j), ...
                Q(i,j),'no-orth');
            VV = A(rprtc{i},cprtc{j})'*UU;
            U{i}(:,rind(i)+1:rind(i) + K(i,j) + P(i,j) ) = UU;
            V{j}(:,cind(j)+1:cind(j) + K(i,j) + P(i,j) ) = VV;
            rind(i) = rind(i) + K(i,j) + P(i,j);
            cind(j) = cind(j) + K(i,j) + P(i,j);
        end
    end
end

for i = 1:rnoc
    [U{i},~] = qr(U{i},0);
end
for i = 1:cnoc
    [V{i},~] = qr(V{i},0);
end

S = compCoreNonsymCell(A,U,[],V,rprtc,cprtc,'dense');




