function [U,S,V] = clustSvdsExt(A,K,prtc,str)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of a square non-symmetric matrix
% A using SVDs of all dense blocks Aij.
%
% [U,S,V] = clustSvdsExt(A,K,prtc,str)
%
% Input:
% A     - Sparse square non-symmetric matrix 
% K     - Matrix indicating ranks in the approximations of blocks Aij
%          If K(i,j) = 0, then Aij is not dense, thus not approximated.
%          If K(i,j) = k_ij > 0, then k_ij is the rank in the 
%          approximation of block Aij.
% prtc  - Cell-array containing vertices for each cluster
% str   - String, must be 'svds' or 'lansvd' to indicate algorithm 
%
% Output:
% U     - Cell-array with orthonormal matrices Ui. Each Ui contains 
%         contributions from all dense blocks in the i'th block-row.
% S     - Cell-array with Sij = Ui'*Aij*Vj
% V     - Cell-array with orthonormal matrices Vj. Each Vj contains 
%         contributions from all dense blocks in the j'th block-column.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of clusters
noc = length(prtc);    
rr = sum(K,2);
cc = sum(K,1);

U = cell(noc,1);
V = cell(noc,1);

for i = 1:noc
    U{i} = zeros(size(prtc{i},1),rr(i));
    V{i} = zeros(size(prtc{i},1),cc(i));
end
rind = zeros(noc,1);
cind = zeros(noc,1);

switch str
    case 'svds'
        disp('Computing clustered matrix approximation with SVDS')
        for i = 1:noc
            for j = 1:noc
                if K(i,j) > 0
                    [UU,~,VV] = svds(A(prtc{i},prtc{j}),K(i,j));
                    U{i}(:,rind(i)+1:rind(i) + K(i,j)) = UU;
                    V{j}(:,cind(j)+1:cind(j) + K(i,j)) = VV;
                    rind(i) = rind(i) + K(i,j);
                    cind(j) = cind(j) + K(i,j);
                end
            end
        end
    case 'lansvd'
        disp('Computing clustered matrix approximation with LANSVD')
        for i = 1:noc
            for j = 1:noc
                if K(i,j) > 0
                    [UU,~,VV] = lansvd(A(prtc{i},prtc{j}),K(i,j));
                    U{i}(:,rind(i)+1:rind(i) + K(i,j)) = UU;
                    V{j}(:,cind(j)+1:cind(j) + K(i,j)) = VV;
                    rind(i) = rind(i) + K(i,j);
                    cind(j) = cind(j) + K(i,j);
                end
            end
        end
        
    otherwise
        disp('Valid string options are "svds" and "lansvd".')
end
for i = 1:noc
    [U{i}, ~] = qr(U{i},0);
    [V{i}, ~] = qr(V{i},0);
end

S = compCoreNonsymCell(A,U,[],V,prtc,prtc,'dense');




