function [U,S,V] = clustSvdsExtRectCell(A,K,rprtc,cprtc,str)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of a rectangular matrix 
% A using SVDs of all dense blocks Aij (svds of all dense blocks)
%
% [U,S,V] = clustSvdsExtRectCell(A,K,rprtc,cprtc,str)
%
% Input:
% A     - Sparse rectangular matrix 
% K     - Matrix indicating ranks in the approximations of blocks Aij
%          If K(i,j) = 0, then Aij is not dense, thus not approximated.
%          If K(i,j) = k_ij > 0, then k_ij is the rank in the 
%          approximation of block Aij.
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
% str   - String, must be 'svds' or 'lansvd' to indicate algorithm 
%
% Output:
% U     - Cell-array with orthonormal matrices Uij. Each Uij contains 
%         contribution only from dense block Aij.
% S     - Cell-array with Sij containing top K(i,j) singular values of Aij.
% V     - Cell-array with orthonormal matrices Vij. Each Vij contains 
%         contribution only from dense block Aij.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of row and column clusters
rnoc = length(rprtc);     
cnoc = length(cprtc);     

U = cell(rnoc,cnoc);
V = cell(rnoc,cnoc);
S = cell(rnoc,cnoc);

switch str
    case 'svds'
        disp('Computing clustered low rank approximation with SVDS')
        for i = 1:rnoc
            disp(['Computation for cluster: ' int2str(i), ' of ' int2str(rnoc)])
            for j = 1:cnoc
                if K(i,j) > 0
                    [U{i,j},S{i,j},V{i,j}] = svds(A(rprtc{i},cprtc{j}),K(i,j));
                end
            end
        end
    case 'lansvd'
        disp('Computing clustered low rank approximation with LANSVD')
        for i = 1:rnoc
            disp(['Computation for cluster: ' int2str(i), ' of ' int2str(rnoc)])
            for j = 1:cnoc
                if K(i,j) > 0
                    [U{i,j},S{i,j},V{i,j}] = lansvd(A(rprtc{i},cprtc{j}),K(i,j));
                end
            end
        end
    otherwise
        disp('Valid string options are "svds" and "lansvd".')
end





