function [U,S,V] = clustSvdsRect(A,K,rprtc,cprtc,str)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of rectangular matrix A
%
% [U,S,V] = clustSvdsRect(A,K,rprtc,cprtc,str);
%
% Input:
% A     - Sparse rectangular matrix
% K     - Matrix indicating ranks in the approximations of blocks Aij
%          If K(i,j) = 0, then Aij is not dense, thus not approximated.
%          If K(i,j) = k_ij > 0, then k_ij is the rank in the
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
% str   - string, must be 'svds' or 'lansvd' to indicate algorithm
%
% Output:
% U     - Cell-array with blocks of U = diag(U_1,...,U_r)
% S     - Cell-array with S_ij = U_i'*A_ij*V_j
% V     - Cell-array with blocks of V = diag(V_1,...,V_c)
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of row anc column clusters
rnoc = length(rprtc);
cnoc = length(cprtc);

rr = sum(K,2);
cc = sum(K,1);

U = cell(rnoc,1);
V = cell(cnoc,1);

for i = 1:rnoc
    U{i} = zeros(size(rprtc{i},1),rr(i));
end

for i = 1:cnoc
    V{i} = zeros(size(cprtc{i},1),cc(i));
end

rind = zeros(rnoc,1);
cind = zeros(cnoc,1);

switch str
    case 'svds'
        disp('Computing clustered low rank approximation with SVDS')
        for i = 1:rnoc
            for j = 1:cnoc
                if K(i,j) > 0
                    disp(['Computation for block: ' int2str(i),' x ',...
                        int2str(j),' of ', int2str(rnoc), ' x ', int2str(cnoc)])
                    [UU,~,VV] = svds(A(rprtc{i},cprtc{j}),K(i,j));
                    U{i}(:,rind(i)+1:rind(i) + K(i,j)) = UU;
                    V{j}(:,cind(j)+1:cind(j) + K(i,j)) = VV;
                    rind(i) = rind(i) + K(i,j);
                    cind(j) = cind(j) + K(i,j);
                end
            end
        end
    case 'lansvd'
        disp('Computing clustered low rank approximation with LANSVD')
        for i = 1:rnoc
            for j = 1:cnoc
                if K(i,j) > 0
                    disp(['Computation for block: ' int2str(i),' x ',...
                        int2str(j),' of ', int2str(rnoc), ' x ', int2str(cnoc)])
                    [UU,~,VV] = lansvd(A(rprtc{i},cprtc{j}),K(i,j));
                    U{i}(:,rind(i)+1:rind(i) + K(i,j)) = UU;
                    V{j}(:,cind(j)+1:cind(j) + K(i,j)) = VV;
                    rind(i) = rind(i) + K(i,j);
                    cind(j) = cind(j) + K(i,j);
                end
            end
        end
        
    otherwise
        disp('Valid string options are "svds" and "lansvd".')
end

for i = 1:rnoc
    [U{i}, ~] = qr(U{i},0);
end
for i = 1:cnoc
    [V{i}, ~] = qr(V{i},0);
end

S = compCoreNonsymCell(A,U,[],V,rprtc,cprtc,'dense');




