function [U,S,V] = clustSvdsRectDiag(A,rprtc,cprtc,k,str)
%--------------------------------------------------------------------------
% Compute clustered matrix approximation of a rectangular matrix A
% using SVD of each diagonal block A_ii.
%
% [U,S,V] = clustSvdsRectDiag(A,rprtc,cprtc,k,str)
%
% Input:
% A     - Sparse rectangular matrix 
% rprtc - Cell-array containing row vertices for each cluster
% cprtc - Cell-array containing column vertices for each cluster
%         rprtc and cprtc must be of same length. Also, the resulting
%         blockpartitioning needs to have dense diagonal blocks. 
% k     - Scalar or vector indicating rank of cluster approximations
%         If k is scalar then the same k is used for all clusters. 
%         If k is vector, then length(k) must equal number of clusters.
% str   - string, must be 'svds' or 'lansvd' to indicate algorithm 
%
% Output:
% U     - Cell-array with top k(i) left singular vectors of each cluster.
% S     - Cell-array with top k(i) singular values of each cluster.
% V     - Cell-array with top k(i) right singular vectors of each cluster.
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

% number of clusters
noc = length(rprtc);    
U = cell(noc,1);
S = cell(noc,1);
V = cell(noc,1);

if length(k) == 1;
    k = ones(noc,1)*k;
end

switch str
    case 'svds'
        disp('Computing clustered low rank approximation with SVDS')
        for i = 1:noc
            rind = rprtc{i};
            cind = cprtc{i};
            disp(['Computation for cluster: ' int2str(i), ' of ' int2str(noc)])
            [U{i},S{i},V{i}] = svds(A(rind,cind),k(i));
        end
    case 'lansvd'
        disp('Computing clustered low rank approximation with LANSVD')
        for i = 1:noc
            rind = rprtc{i};
            cind = cprtc{i};
            disp(['Computation for cluster: ' int2str(i), ' of ' int2str(noc)])
            [U{i},S{i},V{i}] = lansvd(A(rind,cind),k(i));
        end
    otherwise
        disp('Valid string options are "svds" and "lansvd".')
end





