function SS = compCoreNonsymFull(A,U,S,V,prtc,str)
%--------------------------------------------------------------------------
% Compute core/middle matrix in the clustered matrix approximation
%
% SS = compCoreNonsymFull(A,U,S,V,prtc,str)
% 
% Input:
% A     - Sparse square non-symmetric matrix 
% U     - Cell-array with the left  orthonormal matrices for each cluster.
% V     - Cell-array with the right orthonormal matrices for each cluster.
% S     - Optional cell-array with the top k singular values of each cluster.
%           Use [] when S is omitted.
% prtc  - Cell-array containing vertices for each cluster
% str   - string, must be 'diag' or 'dense', 
%           If str = 'diag' then diagonal blocks of SS are given by S.
%           If str = 'dense' then diagonal blocks of SS are computed.
%
% Output: 
% SS    - The core/middle matrix in the clustered matrix approximation
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

%% Get the rank of each cluster approximation.
noc = length(prtc);
k = zeros(noc,1);
for i = 1:noc
    k(i) = size(V{i},2);
end

%% Get row and column index bounds for the core matrix
ind2 = zeros(noc,1);
for i = 1:noc
    ind2(i) = sum(k(1:i));
end
ind1 = ind2 - k+1;

switch str
    case 'diag'
        SS = func1(A,U,S,V,prtc,k,noc,ind1,ind2);
    case 'dense'
        SS = func2(A,U,V,prtc,k,noc,ind1,ind2);
    otherwise
        disp('The str argument must be "diag" or "dense".')
end

% Function for the case str = 'diag'
function SS = func1(A,U,S,V,prtc,k,noc,ind1,ind2)

SS = zeros(sum(k));
for i = 1:noc
    for j = 1:noc
        if i == j
            T = S{i};
        else
            T = U{i}'*A(prtc{i},prtc{j})*V{j};
        end
        SS(ind1(i):ind2(i),ind1(j):ind2(j))  = T;
    end
end

% Function for the case str = 'dense'
function SS = func2(A,U,V,prtc,k,noc,ind1,ind2)
SS = zeros(sum(k));
for i = 1:noc
    for j = 1:noc
        T = U{i}'*A(prtc{i},prtc{j})*V{j};
        SS(ind1(i):ind2(i),ind1(j):ind2(j))  = T;
    end
end

