function DD = compCoreSymFull(A,V,D,prtc,str)
%--------------------------------------------------------------------------
% Compute core matrix in symmetric clustered matrix approximation
%
% DD = compCoreSymFull(A,V,D,prtc,str)
%
% Input:
% A     - Sparse symmetric matrix
% V     - Cell-array with cluster-wise orthonormal matrices
% D     - Cell-array with with Di = Vi'*Aii*Vi 
%           Optional, use [] when D is omitted.
% prtc  - Cell-array containing vertices for each cluster
% str   - string, must be 'diag' or 'dense',
%           If str = 'diag' then diagonal blocks of DD are given by D.
%           If str = 'dense' then diagonal blocks of DD are computed.
%
% Output:
% DD    - The core/middle matrix in the clustered low rank approximation
%--------------------------------------------------------------------------
% CMAPP: Clustered matrix approximation
% Copyright 2015, Berkant Savas
% University of Texas at Austin 2009-2011, Linkoping University 2011-2015
%--------------------------------------------------------------------------

%% Get the rank of each cluster approximation.
noc = length(prtc);
k = zeros(noc,1);
for i = 1:noc
    k(i) = size(V{i},2);
end

%% Get row and column index bounds for the core matrix
ind2 = zeros(noc,1);
for i = 1:noc
    ind2(i) = sum(k(1:i));
end
ind1 = ind2 - k+1;

switch str
    case 'diag'
        DD = func1(A,V,D,prtc,k,noc,ind1,ind2);
    case 'dense'
        DD = func2(A,V,prtc,k,noc,ind1,ind2);
    otherwise
        disp('The str argument must be "diag" or "dense".')
end

end

%% Function for the case str = 'diag'
function DD = func1(A,V,D,prtc,k,noc,ind1,ind2)

DD = zeros(sum(k));
for i = 1:noc
    ii = prtc{i};
    for j = i+1:noc
        jj = prtc{j};
        T = V{i}'*A(ii,jj)*V{j};
        DD(ind1(i):ind2(i),ind1(j):ind2(j))  = T;
        DD(ind1(j):ind2(j),ind1(i):ind2(i))  = T';
    end
    DD(ind1(i):ind2(i),ind1(i):ind2(i)) = D{i};
end

end

%% Function for the case str = 'dense'
function DD = func2(A,V,prtc,k,noc,ind1,ind2)

DD = zeros(sum(k));
for i = 1:noc
    ii = prtc{i};
    for j = i+1:noc
        jj = prtc{j};
        T = V{i}'*A(ii,jj)*V{j};
        DD(ind1(i):ind2(i),ind1(j):ind2(j))  = T;
        DD(ind1(j):ind2(j),ind1(i):ind2(i))  = T';
    end
    T = V{i}'*A(ii,ii)*V{i};
    DD(ind1(i):ind2(i),ind1(i):ind2(i)) = T;
end
end

